/*
 * This file is part of OpenTTD.
 * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
 * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
 */

/** @file date_func.h Functions related to dates. */

#ifndef DATE_FUNC_H
#define DATE_FUNC_H

#include "date_type.h"

void     SetDate              (Date date, TimeTick daytick);
void     SetTimeFactor        ();
void     ConvertDateToYMD     (Date date, DateTime *datetime);
Date     ConvertYMDToDate     (Year year, Month month, Day day);
void     ConvertTimeTickToHM  (TimeTick timetick, DateTime *datetime);
void     ConvertDayTickToHM   (TimeTick daytick, DateTime *datetime);
TimeTick ConvertHMToDayTick   (Hour hour, Minute minute);
void     ConvertNumberToHM    (uint16 *number, Hour *hour, Minute *minute);
void     ConvertNumberToAbsHM (uint16 *number, Hour *hour, Minute *minute);
void     ConvertHMToNumber    (uint16 *number, Hour *hour, Minute *minute);

/**
 * Checks whether the given year is a leap year or not.
 * @param yr The year to check.
 * @return True if \c yr is a leap year, otherwise false.
 */
static inline bool IsLeapYear(Year yr)
{
	return yr % 4 == 0 && (yr % 100 != 0 || yr % 400 == 0);
}

#endif /* DATE_FUNC_H */
