/*
 * This file is part of OpenTTD.
 * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
 * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
 */

/** @file vehicle_orders.h Functions relying both on vehicle and order objects. */



#include "vehicle_orders.h"

#include "depot_base.h"
#include "station_base.h"



/**
 * Returns a tile somewhat representing the order destination (not suitable for pathfinding).
 * @param v The vehicle to get the location for.
 * @param airport Get the airport tile and not the station location for aircraft.
 * @return destination of order, or INVALID_TILE if none.
 */

TileIndex GetLocation (
	const Vehicle *vehicle, const Order *order, bool airport
) {

	switch (order->GetType()) {
		case OT_GOTO_WAYPOINT:
		case OT_GOTO_STATION:
			if (airport && vehicle->type == VEH_AIRCRAFT) return Station::Get(order->dest)->airport.tile;
			return BaseStation::Get(order->dest)->xy;

		case OT_GOTO_DEPOT:
			if ((order->GetDepotActionType() & ODATFB_NEAREST_DEPOT) != 0) return INVALID_TILE;
			return (vehicle->type == VEH_AIRCRAFT) ? Station::Get(order->dest)->xy : Depot::Get(order->dest)->xy;

		default:
			return INVALID_TILE;
	}
}
