/*
 * This file is part of OpenTTD.
 * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
 * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
 */

/** @file follow_track.hpp Template function for track followers */



#ifndef  FOLLOW_TRACK_WATER_HPP
#define  FOLLOW_TRACK_WATER_HPP



#include "follow_track_type.h"
#include "../vehicle/ship.h"



/**
 * Track follower helper template class (can serve pathfinders and vehicle
 *  controllers). See 6 different typedefs below for 3 different transport
 *  types w/ or w/o 90-deg turns allowed
 */
struct CFollowTrackWater
{
	const Ship         *m_veh;           ///< moving vehicle
	Owner               m_veh_owner;     ///< owner of the vehicle
	TileIndex           m_old_tile;      ///< the origin (vehicle moved from) before move
	Trackdir            m_old_td;        ///< the trackdir (the vehicle was on) before move
	TileIndex           m_new_tile;      ///< the new tile (the vehicle has entered)
	TrackdirBits        m_new_td_bits;   ///< the new set of available trackdirs
	DiagDirection       m_exitdir;       ///< exit direction (leaving the old tile)
	bool                m_is_tunnel;     ///< last turn passed tunnel
	bool                m_is_bridge;     ///< last turn passed bridge ramp
	bool                m_is_station;    ///< last turn passed station
	int                 m_tiles_skipped; ///< number of skipped tunnel or station tiles
	ErrorCode           m_err;
	RailTypes           m_railtypes;

	CFollowTrackWater(const Ship *v = nullptr, RailTypes railtype_override = INVALID_RAILTYPES)
	{
		Init(v, railtype_override);
	}

	CFollowTrackWater(Owner o, RailTypes railtype_override = INVALID_RAILTYPES)
	{
		m_veh = nullptr;
		Init(o, railtype_override);
	}

	void Init(const Ship *v, RailTypes railtype_override);
	void Init(Owner o, RailTypes railtype_override);
};



bool Follow(CFollowTrackWater *ftwater, TileIndex old_tile, Trackdir old_td);



#endif /* FOLLOW_TRACK_WATER_HPP */
