/*
 * This file is part of OpenTTD.
 * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
 * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
 */

/** @file timetable.h Functions related to time tabling. */

#ifndef TIMETABLE_H
#define TIMETABLE_H

#include "date_type.h"
#include "vehicle_type.h"

enum TimetableFlag {
	ARRIVAL,
	DEPARTURE
};

/** Container for the arrival/departure dates of a vehicle */
struct TimetableArrivalDeparture {
	TimeTick scheduled_arrival;   ///< The scheduled arrival date
	TimeTick scheduled_departure; ///< The scheduled departure date
	TimeTick arrival;             ///< The actual arrival date
	TimeTick departure;           ///< The actual departure date
};
/*
struct Timetable {
	int      timetable_id;        ///< Same number as order_id.
	TimeTick timed_arrival;       ///< The time of actual arrival at the destination.
	TimeTick timed_departure;     ///< The time of actual departure.
	TimeTick scheduled_arrival;   ///< The time of scheduled arrival.
	TimeTick scheduled_departure; ///< The time of scheduled departure.
	TimeTick wait_time;           ///< How long in ticks to wait at the destination.
	TimeTick travel_time;         ///< How long in ticks the journey to this destination should take.
};
*/
enum timetable_times {
	TA,   ///< timed arrival
	TD,   ///< timed departure
	SA,   ///< scheduled arrival
	SD,   ///< scheduled departure
	WT,   ///< wait time
	TT    ///< total trip time
};

//typedef std::vector<Timetable> TimetableList;

void ShowTimetableWindow    (const Vehicle *v);
void UpdateVehicleTimetable (Vehicle *v, TimetableFlag flag);
void SetTimetableParams     (int param1, int param2, TimeTick ticks);
void TimetableAddTimeSlot   (Vehicle *v, TimeTick time);

#endif /* TIMETABLE_H */
