/*
 * This file is part of OpenTTD.
 * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
 * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
 */

/** @file src/roadveh_static.h Road vehicle states */

#ifndef ROADVEH_STATIC_H
#define ROADVEH_STAIC_H



#include "../direction_type.h"
#include "../road_type.h"
#include "../track_type.h"
#include "../tile_type.h"
#include "vehicle_type.h"



static const uint16 _roadveh_images[] = {
	0xCD4, 0xCDC, 0xCE4, 0xCEC, 0xCF4, 0xCFC, 0xD0C, 0xD14,
	0xD24, 0xD1C, 0xD2C, 0xD04, 0xD1C, 0xD24, 0xD6C, 0xD74,
	0xD7C, 0xC14, 0xC1C, 0xC24, 0xC2C, 0xC34, 0xC3C, 0xC4C,
	0xC54, 0xC64, 0xC5C, 0xC6C, 0xC44, 0xC5C, 0xC64, 0xCAC,
	0xCB4, 0xCBC, 0xD94, 0xD9C, 0xDA4, 0xDAC, 0xDB4, 0xDBC,
	0xDCC, 0xDD4, 0xDE4, 0xDDC, 0xDEC, 0xDC4, 0xDDC, 0xDE4,
	0xE2C, 0xE34, 0xE3C, 0xC14, 0xC1C, 0xC2C, 0xC3C, 0xC4C,
	0xC5C, 0xC64, 0xC6C, 0xC74, 0xC84, 0xC94, 0xCA4
};



static const uint16 _roadveh_full_adder[] = {
	 0,  88,   0,   0,   0,   0,  48,  48,
	48,  48,   0,   0,  64,  64,   0,  16,
	16,   0,  88,   0,   0,   0,   0,  48,
	48,  48,  48,   0,   0,  64,  64,   0,
	16,  16,   0,  88,   0,   0,   0,   0,
	48,  48,  48,  48,   0,   0,  64,  64,
	 0,  16,  16,   0,   8,   8,   8,   8,
	 0,   0,   0,   8,   8,   8,   8
};

static_assert(lengthof(_roadveh_images) == lengthof(_roadveh_full_adder));



static const Trackdir _road_reverse_table[DIAGDIR_END] = {
	TRACKDIR_RVREV_NE, TRACKDIR_RVREV_SE, TRACKDIR_RVREV_SW, TRACKDIR_RVREV_NW
};



static const int8 _delta_xy_table[8][10] = {
	/* y_extent, x_extent, y_offs, x_offs, y_bb_offs, x_bb_offs, y_extent_shorten, x_extent_shorten, y_bb_offs_shorten, x_bb_offs_shorten */
	{3, 3, -1, -1,  0,  0, -1, -1, -1, -1}, // N
	{3, 7, -1, -3,  0, -1,  0, -1,  0,  0}, // NE
	{3, 3, -1, -1,  0,  0,  1, -1,  1, -1}, // E
	{7, 3, -3, -1, -1,  0,  0,  0,  1,  0}, // SE
	{3, 3, -1, -1,  0,  0,  1,  1,  1,  1}, // S
	{3, 7, -1, -3,  0, -1,  0,  0,  0,  1}, // SW
	{3, 3, -1, -1,  0,  0, -1,  1, -1,  1}, // W
	{7, 3, -3, -1, -1,  0, -1,  0,  0,  0}, // NW
};



struct RoadVehFindData {
	int x;
	int y;
	const Vehicle *veh;
	Vehicle *best;
	uint best_diff;
	Direction dir;
};



struct OvertakeData {
	const RoadVehicle *u;
	const RoadVehicle *v;
	TileIndex tile;
	Trackdir trackdir;
};



static const DirDiff delta[] = {
	DIRDIFF_45LEFT, DIRDIFF_SAME, DIRDIFF_SAME, DIRDIFF_45RIGHT
};



static const int8 dist_x[] = { -4, -8, -4, -1, 4, 8, 4, 1 };
static const int8 dist_y[] = { -4, -1, 4, 8, 4, 1, -4, -8 };



static const Direction _roadveh_new_dir[] = {
	DIR_N , DIR_NW, DIR_W , INVALID_DIR,
	DIR_NE, DIR_N , DIR_SW, INVALID_DIR,
	DIR_E , DIR_SE, DIR_S
};



/** Next table is build in the following way:
 *  - first row for when the vehicle in front went to the northern or
 *    western tile, second for southern and eastern.
 *  - columns represent the entry direction.
 *  - cell values are determined by the Trackdir one has to take from
 *    the entry dir (column) to the tile in north or south by only
 *    going over the trackdirs used for turning 90 degrees, i.e.
 *    TRACKDIR_{UPPER,RIGHT,LOWER,LEFT}_{N,E,S,W}.
 */
static const Trackdir reversed_turn_lookup[2][DIAGDIR_END] = {
	{ TRACKDIR_UPPER_W, TRACKDIR_RIGHT_N, TRACKDIR_LEFT_N,  TRACKDIR_UPPER_E },
	{ TRACKDIR_RIGHT_S, TRACKDIR_LOWER_W, TRACKDIR_LOWER_E, TRACKDIR_LEFT_S  }
};



static const RoadBits required_roadbits[] = {
	ROAD_X,            ROAD_Y,            ROAD_NW | ROAD_NE, ROAD_SW | ROAD_SE,
	ROAD_NW | ROAD_SW, ROAD_NE | ROAD_SE, ROAD_X,            ROAD_Y
};



#endif /* ROADVEH_STATIC_H */
